#include <jni.h>
#include <android/log.h>
#include <GLES2/gl2.h>
#include <GLES2/gl2ext.h>
#include <math.h>

GLuint *mTexture;
unsigned int m_texture;
GLuint gProgram;
GLuint gvPositionHandle;
GLuint gvTexCoorHandle;
const GLfloat gTriangleVertices[] = { 0.0f, 0.5f, -0.5f, -0.5f, 0.5f, -0.5f };
const GLfloat gTexCoor[] = { 0.5f, 0, 0, 1, 1, 1 };

static const char gVertexShader[] = "attribute vec4 vPosition;\n"
		"attribute vec2 vTexCoords;\n"
		"varying vec2 colorVarying;\n"
		"void main() {\n"
		"  gl_Position = vPosition;\n"
		"  colorVarying = vTexCoords;\n"
		"}\n";

static const char gFragmentShader[] = "precision mediump float;\n"
		"varying vec2 colorVarying;\n"
		"uniform sampler2D sampler;\n"
		"void main() {\n"
		"  //gl_FragColor = vec4(0.0, 1.0, 0.0, 1.0);\n"
		"gl_FragColor = texture2D(sampler,colorVarying);\n"
		"}\n";

GLuint loadShader(GLenum shaderType, const char* pSource) {
	GLuint shader = glCreateShader(shaderType);
	if (shader) {
		glShaderSource(shader, 1, &pSource, NULL);
		glCompileShader(shader);
		GLint compiled = 0;
		glGetShaderiv(shader, GL_COMPILE_STATUS, &compiled);
		if (!compiled) {
			GLint infoLen = 0;
			glGetShaderiv(shader, GL_INFO_LOG_LENGTH, &infoLen);
			if (infoLen) {
				char* buf = new char[infoLen];
				if (buf) {
					glGetShaderInfoLog(shader, infoLen, NULL, buf);
					delete buf;
				}
				glDeleteShader(shader);
				shader = 0;
			}
		}
	}
	return shader;
}

GLuint createProgram(const char* pVertexSource, const char* pFragmentSource) {
	GLuint vertexShader = loadShader(GL_VERTEX_SHADER, pVertexSource);
	if (!vertexShader) {
		return 0;
	}
	GLuint pixelShader = loadShader(GL_FRAGMENT_SHADER, pFragmentSource);
	if (!pixelShader) {
		return 0;
	}
	GLuint program = glCreateProgram();
	if (program) {
		glAttachShader(program, vertexShader);
		glAttachShader(program, pixelShader);
		glLinkProgram(program);
		GLint linkStatus = GL_FALSE;
		glGetProgramiv(program, GL_LINK_STATUS, &linkStatus);
		if (linkStatus != GL_TRUE) {
			GLint bufLength = 0;
			glGetProgramiv(program, GL_INFO_LOG_LENGTH, &bufLength);
			if (bufLength) {
				char* buf = new char[bufLength];
				if (buf) {
					glGetProgramInfoLog(program, bufLength, NULL, buf);
					delete buf;
				}
			}
			glDeleteProgram(program);
			program = 0;
		}
	}
	return program;
}

bool setupGraphics(int w, int h) {
	gProgram = createProgram(gVertexShader, gFragmentShader);
	if (!gProgram) {
		return false;
	}
	gvPositionHandle = glGetAttribLocation(gProgram, "vPosition");
	gvTexCoorHandle = glGetAttribLocation(gProgram, "vTexCoords");
	glViewport(0, 0, w, h);
	return true;
}

void renderFrame() {
	glClearColor(1.0f, 1.0f, 1.0f, 1.0f);
	glClear(GL_DEPTH_BUFFER_BIT | GL_COLOR_BUFFER_BIT);
	//glClear( GL_STENCIL_BUFFER_BIT | GL_COLOR_BUFFER_BIT);
	glUseProgram(gProgram);
	glVertexAttribPointer(gvPositionHandle, 2, GL_FLOAT, GL_FALSE, 0,
			gTriangleVertices);
	glVertexAttribPointer(gvTexCoorHandle, 2, GL_FLOAT, GL_FALSE, 0, gTexCoor);
	glEnableVertexAttribArray(gvPositionHandle);
	glEnableVertexAttribArray(gvTexCoorHandle);
	glActiveTexture(GL_TEXTURE0);
	glBindTexture(GL_TEXTURE_2D, mTexture[0]);
	glDrawArrays(GL_TRIANGLES, 0, 3);
}

extern "C" JNIEXPORT void JNICALL Java_com_android_gl2jni_GL2JNILib_init(
		JNIEnv * env, jobject obj, jint width, jint height) {
	setupGraphics(width, height);
}

extern "C" JNIEXPORT void JNICALL Java_com_android_gl2jni_GL2JNILib_step(
		JNIEnv * env, jobject obj) {
	renderFrame();
}

extern "C" JNIEXPORT void JNICALL Java_com_android_gl2jni_GL2JNILib_setTextures(
		JNIEnv * env, jobject obj, jintArray texture) {
	mTexture = (GLuint *) env->GetIntArrayElements(texture, 0);
}
